@@ -18,7 +18,7 @@ from ..errors import (
     MissingBands,
     TileOutsideBounds,
 )
-from ..expression import apply_expression
+from ..expression import apply_expression, get_expression_blocks
 from ..models import BandStatistics, ImageData, Info
 from ..tasks import multi_arrays, multi_values
 from ..types import BBox, Indexes
@@ -373,7 +373,7 @@ class MultiBaseReader(SpatialMixin, metaclass=abc.ABCMeta):
 
     def parse_expression(self, expression: str) -> Tuple:
         """Parse rio-tiler band math expression."""
-        assets = "|".join([fr"\b{asset}\b" for asset in self.assets])
+        assets = "|".join([rf"\b{asset}\b" for asset in self.assets])
         _re = re.compile(assets.replace("\\\\", "\\"))
         return tuple(set(re.findall(_re, expression)))
 
@@ -445,13 +445,77 @@ class MultiBaseReader(SpatialMixin, metaclass=abc.ABCMeta):
             with self.reader(url, tms=self.tms, **self.reader_options) as cog:  # type: ignore
                 return cog.statistics(
                     *args,
-                    indexes=asset_indexes.get(asset, kwargs.pop("indexes", None)),
-                    expression=asset_expression.get(asset),
+                    indexes=asset_indexes.get(asset, kwargs.pop("indexes", None)),  # type: ignore
+                    expression=asset_expression.get(asset),  # type: ignore
                     **kwargs,
                 )
 
         return multi_values(assets, _reader, **kwargs)
 
+    def merged_statistics(  # type: ignore
+        self,
+        assets: Union[Sequence[str], str] = None,
+        expression: Optional[str] = None,
+        asset_indexes: Optional[Dict[str, Indexes]] = None,  # Indexes for each asset
+        asset_expression: Optional[Dict[str, str]] = None,  # Expression for each asset
+        categorical: bool = False,
+        categories: Optional[List[float]] = None,
+        percentiles: List[int] = [2, 98],
+        hist_options: Optional[Dict] = None,
+        max_size: int = 1024,
+        **kwargs: Any,
+    ) -> Dict[str, BandStatistics]:
+        """Return array statistics for multiple assets.
+
+        Args:
+            assets (sequence of str or str): assets to fetch info from.
+            expression (str, optional): rio-tiler expression for the asset list (e.g. asset1/asset2+asset3).
+            asset_indexes (dict, optional): Band indexes for each asset (e.g {"asset1": 1, "asset2": (1, 2,)}).
+            asset_expression (dict, optional): rio-tiler expression for each asset (e.g. {"asset1": "b1/b2+b3", "asset2": ...}).
+            categorical (bool): treat input data as categorical data. Defaults to False.
+            categories (list of numbers, optional): list of categories to return value for.
+            percentiles (list of numbers, optional): list of percentile values to calculate. Defaults to `[2, 98]`.
+            hist_options (dict, optional): Options to forward to numpy.histogram function.
+            max_size (int, optional): Limit the size of the longest dimension of the dataset read, respecting bounds X/Y aspect ratio. Defaults to 1024.
+            kwargs (optional): Options to forward to the `self.preview` method.
+
+
+        Returns:
+            Dict[str, rio_tiler.models.BandStatistics]: bands statistics.
+
+        """
+        if not expression:
+            if not assets:
+                warnings.warn(
+                    "No `assets` option passed, will fetch statistics for all available assets.",
+                    UserWarning,
+                )
+            assets = assets or self.assets
+
+        data = self.preview(
+            assets=assets,
+            expression=expression,
+            asset_indexes=asset_indexes,
+            asset_expression=asset_expression,
+            max_size=max_size,
+            **kwargs,
+        )
+
+        hist_options = hist_options or {}
+
+        stats = get_array_statistics(
+            data.as_masked(),
+            categorical=categorical,
+            categories=categories,
+            percentiles=percentiles,
+            **hist_options,
+        )
+
+        return {
+            f"{data.band_names[ix]}": BandStatistics(**stats[ix])
+            for ix in range(len(stats))
+        }
+
     def tile(
         self,
         tile_x: int,
@@ -509,8 +573,8 @@ class MultiBaseReader(SpatialMixin, metaclass=abc.ABCMeta):
             with self.reader(url, tms=self.tms, **self.reader_options) as cog:  # type: ignore
                 data = cog.tile(
                     *args,
-                    indexes=asset_indexes.get(asset, kwargs.pop("indexes", None)),
-                    expression=asset_expression.get(asset),
+                    indexes=asset_indexes.get(asset, kwargs.pop("indexes", None)),  # type: ignore
+                    expression=asset_expression.get(asset),  # type: ignore
                     **kwargs,
                 )
                 data.band_names = [f"{asset}_{n}" for n in data.band_names]
@@ -526,7 +590,7 @@ class MultiBaseReader(SpatialMixin, metaclass=abc.ABCMeta):
         )
 
         if expression:
-            blocks = expression.split(",")
+            blocks = get_expression_blocks(expression)
             output.data = apply_expression(blocks, assets, output.data)
             output.band_names = blocks
 
@@ -580,8 +644,8 @@ class MultiBaseReader(SpatialMixin, metaclass=abc.ABCMeta):
             with self.reader(url, tms=self.tms, **self.reader_options) as cog:  # type: ignore
                 data = cog.part(
                     *args,
-                    indexes=asset_indexes.get(asset, kwargs.pop("indexes", None)),
-                    expression=asset_expression.get(asset),
+                    indexes=asset_indexes.get(asset, kwargs.pop("indexes", None)),  # type: ignore
+                    expression=asset_expression.get(asset),  # type: ignore
                     **kwargs,
                 )
                 data.band_names = [f"{asset}_{n}" for n in data.band_names]
@@ -590,7 +654,7 @@ class MultiBaseReader(SpatialMixin, metaclass=abc.ABCMeta):
         output = multi_arrays(assets, _reader, bbox, **kwargs)
 
         if expression:
-            blocks = expression.split(",")
+            blocks = get_expression_blocks(expression)
             output.data = apply_expression(blocks, assets, output.data)
             output.band_names = blocks
 
@@ -641,8 +705,8 @@ class MultiBaseReader(SpatialMixin, metaclass=abc.ABCMeta):
             url = self._get_asset_url(asset)
             with self.reader(url, tms=self.tms, **self.reader_options) as cog:  # type: ignore
                 data = cog.preview(
-                    indexes=asset_indexes.get(asset, kwargs.pop("indexes", None)),
-                    expression=asset_expression.get(asset),
+                    indexes=asset_indexes.get(asset, kwargs.pop("indexes", None)),  # type: ignore
+                    expression=asset_expression.get(asset),  # type: ignore
                     **kwargs,
                 )
                 data.band_names = [f"{asset}_{n}" for n in data.band_names]
@@ -651,7 +715,7 @@ class MultiBaseReader(SpatialMixin, metaclass=abc.ABCMeta):
         output = multi_arrays(assets, _reader, **kwargs)
 
         if expression:
-            blocks = expression.split(",")
+            blocks = get_expression_blocks(expression)
             output.data = apply_expression(blocks, assets, output.data)
             output.band_names = blocks
 
@@ -707,8 +771,8 @@ class MultiBaseReader(SpatialMixin, metaclass=abc.ABCMeta):
             with self.reader(url, tms=self.tms, **self.reader_options) as cog:  # type: ignore
                 return cog.point(
                     *args,
-                    indexes=asset_indexes.get(asset, kwargs.pop("indexes", None)),
-                    expression=asset_expression.get(asset),
+                    indexes=asset_indexes.get(asset, kwargs.pop("indexes", None)),  # type: ignore
+                    expression=asset_expression.get(asset),  # type: ignore
                     **kwargs,
                 )
 
@@ -716,7 +780,7 @@ class MultiBaseReader(SpatialMixin, metaclass=abc.ABCMeta):
 
         values = numpy.array([d for _, d in data.items()])
         if expression:
-            blocks = expression.split(",")
+            blocks = get_expression_blocks(expression)
             values = apply_expression(blocks, assets, values)
 
         return values.tolist()
@@ -769,8 +833,8 @@ class MultiBaseReader(SpatialMixin, metaclass=abc.ABCMeta):
             with self.reader(url, tms=self.tms, **self.reader_options) as cog:  # type: ignore
                 data = cog.feature(
                     *args,
-                    indexes=asset_indexes.get(asset, kwargs.pop("indexes", None)),
-                    expression=asset_expression.get(asset),
+                    indexes=asset_indexes.get(asset, kwargs.pop("indexes", None)),  # type: ignore
+                    expression=asset_expression.get(asset),  # type: ignore
                     **kwargs,
                 )
                 data.band_names = [f"{asset}_{n}" for n in data.band_names]
@@ -779,7 +843,7 @@ class MultiBaseReader(SpatialMixin, metaclass=abc.ABCMeta):
         output = multi_arrays(assets, _reader, shape, **kwargs)
 
         if expression:
-            blocks = expression.split(",")
+            blocks = get_expression_blocks(expression)
             output.data = apply_expression(blocks, assets, output.data)
             output.band_names = blocks
 
@@ -823,7 +887,7 @@ class MultiBandReader(SpatialMixin, metaclass=abc.ABCMeta):
 
     def parse_expression(self, expression: str) -> Tuple:
         """Parse rio-tiler band math expression."""
-        bands = "|".join([fr"\b{band}\b" for band in self.bands])
+        bands = "|".join([rf"\b{band}\b" for band in self.bands])
         _re = re.compile(bands.replace("\\\\", "\\"))
         return tuple(set(re.findall(_re, expression)))
 
@@ -991,7 +1055,7 @@ class MultiBandReader(SpatialMixin, metaclass=abc.ABCMeta):
         output = multi_arrays(bands, _reader, tile_x, tile_y, tile_z, **kwargs)
 
         if expression:
-            blocks = expression.split(",")
+            blocks = get_expression_blocks(expression)
             output.data = apply_expression(blocks, bands, output.data)
             output.band_names = blocks
 
@@ -1043,7 +1107,7 @@ class MultiBandReader(SpatialMixin, metaclass=abc.ABCMeta):
         output = multi_arrays(bands, _reader, bbox, **kwargs)
 
         if expression:
-            blocks = expression.split(",")
+            blocks = get_expression_blocks(expression)
             output.data = apply_expression(blocks, bands, output.data)
             output.band_names = blocks
 
@@ -1093,7 +1157,7 @@ class MultiBandReader(SpatialMixin, metaclass=abc.ABCMeta):
         output = multi_arrays(bands, _reader, **kwargs)
 
         if expression:
-            blocks = expression.split(",")
+            blocks = get_expression_blocks(expression)
             output.data = apply_expression(blocks, bands, output.data)
             output.band_names = blocks
 
@@ -1146,7 +1210,7 @@ class MultiBandReader(SpatialMixin, metaclass=abc.ABCMeta):
 
         values = numpy.array([d for _, d in data.items()])
         if expression:
-            blocks = expression.split(",")
+            blocks = get_expression_blocks(expression)
             values = apply_expression(blocks, bands, values)
 
         return values.tolist()
@@ -1197,7 +1261,7 @@ class MultiBandReader(SpatialMixin, metaclass=abc.ABCMeta):
         output = multi_arrays(bands, _reader, shape, **kwargs)
 
         if expression:
-            blocks = expression.split(",")
+            blocks = get_expression_blocks(expression)
             output.data = apply_expression(blocks, bands, output.data)
             output.band_names = blocks
 
