@@ -25,7 +25,7 @@ from ..errors import (
     NoOverviewWarning,
     TileOutsideBounds,
 )
-from ..expression import apply_expression, parse_expression
+from ..expression import apply_expression, get_expression_blocks, parse_expression
 from ..models import BandStatistics, ImageData, Info
 from ..types import BBox, DataMaskType, Indexes, NoData, NumType
 from ..utils import (
@@ -171,7 +171,7 @@ class COGReader(BaseReader):
 
         # The minzoom is defined by the resolution of the maximum theoretical overview level
         overview_level = get_maximum_overview_level(w, h, minsize=tilesize)
-        ovr_resolution = resolution * (2 ** overview_level)
+        ovr_resolution = resolution * (2**overview_level)
         minzoom = self.tms.zoom_for_res(ovr_resolution)
 
         return (minzoom, maxzoom)
@@ -416,8 +416,8 @@ class COGReader(BaseReader):
             **kwargs,
         )
 
-        if expression:
-            blocks = expression.lower().split(",")
+        if expression and indexes:
+            blocks = get_expression_blocks(expression)
             bands = [f"b{bidx}" for bidx in indexes]
             data = apply_expression(blocks, bands, data)
 
@@ -481,8 +481,8 @@ class COGReader(BaseReader):
             **kwargs,
         )
 
-        if expression:
-            blocks = expression.lower().split(",")
+        if expression and indexes:
+            blocks = get_expression_blocks(expression)
             bands = [f"b{bidx}" for bidx in indexes]
             data = apply_expression(blocks, bands, data)
 
@@ -538,8 +538,8 @@ class COGReader(BaseReader):
             self.dataset, (lon, lat), indexes=indexes, coord_crs=coord_crs, **kwargs
         )
 
-        if expression:
-            blocks = expression.lower().split(",")
+        if expression and indexes:
+            blocks = get_expression_blocks(expression)
             bands = [f"b{bidx}" for bidx in indexes]
             point = apply_expression(blocks, bands, numpy.array(point)).tolist()
 
@@ -630,8 +630,8 @@ class COGReader(BaseReader):
 
         data, mask = reader.read(self.dataset, indexes=indexes, **kwargs)
 
-        if expression:
-            blocks = expression.lower().split(",")
+        if expression and indexes:
+            blocks = get_expression_blocks(expression)
             bands = [f"b{bidx}" for bidx in indexes]
             data = apply_expression(blocks, bands, data)
 
