@@ -22,6 +22,7 @@ from rasterio.warp import calculate_default_transform, transform_geom
 from .colormap import apply_cmap
 from .constants import WEB_MERCATOR_CRS
 from .errors import RioTilerError
+from .expression import get_expression_blocks
 from .types import BBox, ColorMapType, IntervalTuple
 
 
@@ -40,7 +41,15 @@ def aws_get_object(
     """AWS s3 get object content."""
     if not client:
         session = boto3_session()
+        # AWS_S3_ENDPOINT and AWS_HTTPS are GDAL config options of vsis3 driver
+        # https://gdal.org/user/virtual_file_systems.html#vsis3-aws-s3-files
         endpoint_url = os.environ.get("AWS_S3_ENDPOINT", None)
+        if endpoint_url is not None:
+            use_https = os.environ.get("AWS_HTTPS", "YES")
+            if use_https.upper() in ["YES", "TRUE", "ON"]:
+                endpoint_url = "https://" + endpoint_url
+            else:
+                endpoint_url = "http://" + endpoint_url
         client = session.client("s3", endpoint_url=endpoint_url)
 
     params = {"Bucket": bucket, "Key": key}
@@ -58,7 +67,7 @@ def get_bands_names(
 ) -> List[str]:
     """Define bands names based on expression, indexes or band count."""
     if expression:
-        return expression.split(",")
+        return get_expression_blocks(expression)
 
     elif indexes:
         return [str(idx) for idx in indexes]
@@ -83,13 +92,13 @@ def get_array_statistics(
 
     Args:
         data (numpy.ma.MaskedArray): input masked array data to get the statistics from.
-        categorical (bool): treat input data as categorical data. Defaults to False.
+        categorical (bool): treat input data as categorical data. Defaults to `False`.
         categories (list of numbers, optional): list of categories to return value for.
         percentiles (list of numbers, optional): list of percentile values to calculate. Defaults to `[2, 98]`.
         kwargs (optional): options to forward to `numpy.histogram` function (only applies for non-categorical data).
 
     Returns:
-        list of dict
+        list: list of array statistics (dict)
 
     Examples:
         >>> data = numpy.ma.zeros((1, 256, 256))
@@ -125,6 +134,9 @@ def get_array_statistics(
     output: List[Dict[Any, Any]] = []
     percentiles_names = [f"percentile_{int(p)}" for p in percentiles]
 
+    # Avoid non masked nan/inf values
+    numpy.ma.fix_invalid(data, copy=False)
+
     for b in range(data.shape[0]):
         keys, counts = numpy.unique(data[b].compressed(), return_counts=True)
 
@@ -147,7 +159,7 @@ def get_array_statistics(
                 h_keys,
             ]
         else:
-            h_counts, h_keys = numpy.histogram(data[b][~data[b].mask], **kwargs)
+            h_counts, h_keys = numpy.histogram(data[b].compressed(), **kwargs)
             histogram = [h_counts.tolist(), h_keys.tolist()]
 
         percentiles_values = numpy.percentile(
@@ -250,28 +262,33 @@ def get_vrt_transform(
         tuple: VRT transform (affine.Affine), width (int) and height (int)
 
     """
-    dst_transform, _, _ = calculate_default_transform(
-        src_dst.crs, dst_crs, src_dst.width, src_dst.height, *src_dst.bounds
-    )
+    if src_dst.crs != dst_crs:
+        dst_transform, _, _ = calculate_default_transform(
+            src_dst.crs, dst_crs, src_dst.width, src_dst.height, *src_dst.bounds
+        )
+    else:
+        dst_transform = src_dst.transform
 
     # If bounds window is aligned with the dataset internal tile we align the bounds with the pixels.
     # This is to limit the number of internal block fetched.
     if _requested_tile_aligned_with_internal_tile(
         src_dst, bounds, height, width, dst_crs
     ):
+        # Get Window for the input bounds
+        # e.g Window(col_off=17920.0, row_off=11007.999999999998, width=255.99999999999636, height=256.0000000000018)
         col_off, row_off, w, h = windows.from_bounds(
-            *bounds,
-            transform=src_dst.transform,
-            width=width,
-            height=height,
+            *bounds, transform=dst_transform
         ).flatten()
 
+        # Round Window
         w = windows.Window(
             round(col_off, window_precision),
             round(row_off, window_precision),
             round(w, window_precision),
             round(h, window_precision),
         )
+
+        # Get Bounds for the rounded window
         bounds = src_dst.window_bounds(w)
 
     w, s, e, n = bounds
@@ -522,6 +539,22 @@ def pansharpening_brovey(
         return numpy.clip(ratio * rgb, 0, numpy.iinfo(pan_dtype).max).astype(pan_dtype)
 
 
+def _convert_to_raster_space(
+    src_dst: Union[DatasetReader, DatasetWriter, WarpedVRT],
+    poly_coordinates: List,
+) -> List[str]:
+    polygons = []
+    for point in poly_coordinates:
+        xs, ys = zip(*coords(point))
+        src_y, src_x = rowcol(src_dst.transform, xs, ys)
+        src_x = [max(0, min(src_dst.width, x)) for x in src_x]
+        src_y = [max(0, min(src_dst.height, y)) for y in src_y]
+        polygon = ", ".join([f"{x} {y}" for x, y in list(zip(src_x, src_y))])
+        polygons.append(f"({polygon})")
+
+    return polygons
+
+
 def create_cutline(
     src_dst: Union[DatasetReader, DatasetWriter, WarpedVRT],
     geometry: Dict,
@@ -547,33 +580,28 @@ def create_cutline(
         raise RioTilerError("Invalid geometry")
 
     geom_type = geometry["type"]
-    if geom_type not in ["Polygon", "MultiPolygon"]:
-        raise RioTilerError(
-            "Invalid geometry type: {geom_type}. Should be Polygon or MultiPolygon"
-        )
 
     if geometry_crs:
         geometry = transform_geom(geometry_crs, src_dst.crs, geometry)
 
-    polys = []
-    geom = (
-        [geometry["coordinates"]] if geom_type == "Polygon" else geometry["coordinates"]
-    )
-    for p in geom:
-        xs, ys = zip(*coords(p))
-        src_y, src_x = rowcol(src_dst.transform, xs, ys)
-        src_x = [max(0, min(src_dst.width, x)) for x in src_x]
-        src_y = [max(0, min(src_dst.height, y)) for y in src_y]
-        poly = ", ".join([f"{x} {y}" for x, y in list(zip(src_x, src_y))])
-        polys.append(f"(({poly}))")
+    if geom_type == "Polygon":
+        polys = ",".join(_convert_to_raster_space(src_dst, geometry["coordinates"]))
+        wkt = f"POLYGON ({polys})"
 
-    str_poly = ",".join(polys)
+    elif geom_type == "MultiPolygon":
+        multi_polys = []
+        for poly in geometry["coordinates"]:
+            polys = ",".join(_convert_to_raster_space(src_dst, poly))
+            multi_polys.append(f"({polys})")
+        str_multipoly = ",".join(multi_polys)
+        wkt = f"MULTIPOLYGON ({str_multipoly})"
 
-    return (
-        f"POLYGON {str_poly}"
-        if geom_type == "Polygon"
-        else f"MULTIPOLYGON ({str_poly})"
-    )
+    else:
+        raise RioTilerError(
+            "Invalid geometry type: {geom_type}. Should be Polygon or MultiPolygon"
+        )
+
+    return wkt
 
 
 def resize_array(
